const Readable = require('stream').Readable

const isResponse = response =>
  typeof response === 'object' &&
  typeof response.statusCode === 'number' &&
  typeof response.headers === 'object'

const isReadable = response =>
  response.body instanceof Readable

const respond = (httpResponse, resolved = true) => response => {
  if (!resolved)
    console.error('Error:\n', response)

  if (!isResponse(response))
    response = {
      statusCode: resolved ? 200 : 500,
      headers: {},
      body: response
    }

  if (response.body instanceof Error)
    response.body = {
      message: response.body.message
    }

  if (isReadable(response)) {
    httpResponse.writeHead(response.statusCode, Object.assign({
      'Cache-Control': 'no-cache',
      'Connection': 'keep-alive',
      'Content-Type': 'text/event-stream; charset=utf-8'
    }, response.headers))
    response.body.pipe(httpResponse)
  }

  else {
    const responseBody =
      typeof response.body === 'string' ?
        response.body :
        JSON.stringify(response.body) || ''

    httpResponse.writeHead(response.statusCode, Object.assign({
      'Content-Length': Buffer.byteLength(responseBody),
      'Content-Type': 'application/json; charset=utf-8'
    }, response.headers))
    httpResponse.end(responseBody)
  }
}

module.exports = httpResponse => ({
  onResolved: respond(httpResponse),
  onRejected: respond(httpResponse, false)
})
